@@ -38,8 +38,11 @@ def create_float_zeros_as_CFFI_double( n):
     return x, px
 
 def convert_py_float_to_cffi( x):
-    px = np.array( x)
-    if px.dtype != "float64": px = px.astype( float)
+    if isinstance( x, np.ndarray) == True:
+        px = x
+    else:
+        px = np.array( x)
+        if px.dtype != "float64": px = px.astype( float)
     pxcffi = ffi.cast( "double *", px.ctypes.data)
     return px, pxcffi
 
@@ -1041,7 +1044,6 @@ def populate_fm_2add_sparse(singletons, numpairs, pairs, indicesp1, indicesp2, e
     pindicesp1np, pindicesp1 = convert_py_int_to_cffi( indicesp1)
     pindicesp2np, pindicesp2 = convert_py_int_to_cffi( indicesp2)
     fm.py_populate_fm_2add_sparse( psingletons, numpairs, ppairs, pindicesp1, pindicesp2, env)
-    return env
 
 
 # Generated python wrapper for:
@@ -1049,7 +1051,6 @@ def populate_fm_2add_sparse(singletons, numpairs, pairs, indicesp1, indicesp2, e
 def add_pair_sparse(i, j, v, env):
     trace( "void py_add_pair_sparse(int i, int j, double v, struct fm_env_sparse* env)")
     fm.py_add_pair_sparse( i, j, v, env)
-    return env
 
 
 # Generated python wrapper for:
@@ -1058,16 +1059,15 @@ def populate_fm_2add_sparse_from2add(n, v, env):
     trace( "void py_populate_fm_2add_sparse_from2add(int n, double* v, struct fm_env_sparse* env)")
     pvnp, pv = convert_py_float_to_cffi( v)
     fm.py_populate_fm_2add_sparse_from2add( n, pv, env)
-    return env
 
 
 # Generated python wrapper for:
-#    env)void py_expand_2add_full(double* v, struct fm_env_sparse* env)
-def expand_2add_full(v, env):
-    trace( "env)void py_expand_2add_full(double* v, struct fm_env_sparse* env)")
-    pvnp, pv = convert_py_float_to_cffi( v)
-    fm.py_expand_2add_full( pv, env)
-    return env
+#    void py_expand_2add_full(double* out_v, struct fm_env_sparse* env)
+def expand_2add_full(n, env):
+    trace( "void py_expand_2add_full(double* out_v, struct fm_env_sparse* env)")
+    pout_vnp, pout_v = create_float_zeros_as_CFFI_double( 2 ** n)
+    fm.py_expand_2add_full( pout_v, env)
+    return pout_vnp
 
 
 # Generated python wrapper for:
@@ -1076,37 +1076,35 @@ def expand_sparse_full(v, env):
     trace( "void py_expand_sparse_full(double* v, struct fm_env_sparse* env)")
     pvnp, pv = convert_py_float_to_cffi( v)
     fm.py_expand_sparse_full( pv, env)
-    return env
 
 
 # Generated python wrapper for:
-#    void py_sparse_get_singletons(int n, double* v, struct fm_env_sparse* env)
-def sparse_get_singletons(n, v, env):
-    trace( "void py_sparse_get_singletons(int n, double* v, struct fm_env_sparse* env)")
-    pvnp, pv = convert_py_float_to_cffi( v)
-    fm.py_sparse_get_singletons( n, pv, env)
-    return env
-
-
-
-# Generated python wrapper for:
-#    int py_sparse_get_pairs(int* pairs, double* v, struct fm_env_sparse* env)
-def sparse_get_pairs(pairs, v, env):
-    trace( "int py_sparse_get_pairs(int* pairs, double* v, struct fm_env_sparse* env)")
-    ppairsnp, ppairs = convert_py_int_to_cffi( pairs)
-    pvnp, pv = convert_py_float_to_cffi( v)
-    yy = fm.py_sparse_get_pairs( ppairs, pv, env)
-    return yy
+#    void py_sparse_get_singletons(int n, double* out_v, struct fm_env_sparse* env)
+def sparse_get_singletons(n, env):
+    trace( "void py_sparse_get_singletons(int n, double* out_v, struct fm_env_sparse* env)")
+    pout_vnp, pout_v = create_float_zeros_as_CFFI_double( 2 ** n)
+    fm.py_sparse_get_singletons( n, pout_v, env)
+    return pout_vnp
 
 
 # Generated python wrapper for:
-#    int py_sparse_get_tuples(int* tuples, double* v, struct fm_env_sparse* env)
-def sparse_get_tuples(tuples, v, env):
-    trace( "int py_sparse_get_tuples(int* tuples, double* v, struct fm_env_sparse* env)")
-    ptuplesnp, ptuples = convert_py_int_to_cffi( tuples)
-    pvnp, pv = convert_py_float_to_cffi( v)
-    yy = fm.py_sparse_get_tuples( ptuples, pv, env)
-    return yy
+#    int py_sparse_get_pairs(int* out_pairs, double* out_v, struct fm_env_sparse* env)
+def sparse_get_pairs(n, env):
+    trace( "int py_sparse_get_pairs(int* out_pairs, double* out_v, struct fm_env_sparse* env)")
+    pout_pairsnp, pout_pairs = create_intc_zeros_as_CFFI_int( 2 ** n)
+    pout_vnp, pout_v = create_float_zeros_as_CFFI_double( 2 ** n)
+    yy = fm.py_sparse_get_pairs( pout_pairs, pout_v, env)
+    return pout_pairsnp, pout_vnp
+
+
+# Generated python wrapper for:
+#    int py_sparse_get_tuples(int* out_tuples, double* out_v, struct fm_env_sparse* env)
+def sparse_get_tuples(n, env):
+    trace( "int py_sparse_get_tuples(int* out_tuples, double* out_v, struct fm_env_sparse* env)")
+    pout_tuplesnp, pout_tuples = create_intc_zeros_as_CFFI_int( 2 ** n)
+    pout_vnp, pout_v = create_float_zeros_as_CFFI_double( 2 ** n)
+    yy = fm.py_sparse_get_tuples( pout_tuples, pout_v, env)
+    return pout_tuplesnp, pout_vnp
 
 
 # Generated python wrapper for:
